@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.59 1999/10/02 19:52:37 nisse Exp $
+ * $Id: channel.c,v 1.61 1999/10/11 20:34:39 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -241,14 +241,17 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
 static struct exception_handler *
 make_exc_finish_channel_handler(struct channel_table *table,
 				UINT32 channel_number,
-				struct exception_handler *e)
+				struct exception_handler *e,
+				const char *context)
 {
   NEW(exc_finish_channel_handler, self);
-  self->table = table;
-  self->channel_number = channel_number;
   self->super.parent = e;
   self->super.raise = do_exc_finish_channel_handler;
+  self->super.context = context;
 
+  self->table = table;
+  self->channel_number = channel_number;
+  
   return &self->super;
 }
 				
@@ -361,7 +364,8 @@ register_channel(struct ssh_connection *connection,
   /* FIXME: Is this the right place to install this exception handler? */
   channel->e = make_exc_finish_channel_handler(table,
 					       local_channel_number,
-					       connection->e);
+					       connection->e,
+					       HANDLER_CONTEXT);
 
   REMEMBER_RESOURCE(connection->resources, &channel->resources->super);
 }
@@ -1059,8 +1063,7 @@ do_channel_eof(struct packet_handler *closure UNUSED,
 	      else
 		/* FIXME: What is a reasonable default behaviour?
 		 * Closing the channel may be the right thing to do. */
-		if (! (channel->flags & CHANNEL_SENT_CLOSE))
-		  channel_close(channel);
+		channel_close(channel);
 	    }
 	}
       else
@@ -1540,8 +1543,7 @@ do_channel_write(struct abstract_write *w,
     {
       /* EOF */
       assert(closure->channel->sources);
-      if ( (closure->channel->sources == 1)
-	   && !(closure->channel->flags & CHANNEL_SENT_EOF))
+      if (closure->channel->sources == 1)
 	channel_eof(closure->channel);
     }
   else
@@ -1559,8 +1561,7 @@ do_channel_write_extended(struct abstract_write *w,
     {
       /* EOF */
       assert(closure->super.channel->sources);
-      if ( (closure->super.channel->sources == 1)
-	   && !(closure->super.channel->flags & CHANNEL_SENT_EOF))
+      if (closure->super.channel->sources == 1)
 	channel_eof(closure->super.channel);
     }
   else
@@ -1727,11 +1728,14 @@ do_channel_io_exception_handler(struct exception_handler *s,
 struct exception_handler *
 make_channel_io_exception_handler(struct ssh_channel *channel,
 				  const char *prefix,
-				  struct exception_handler *parent)
+				  struct exception_handler *parent,
+				  const char *context)
 {
   NEW(channel_io_exception_handler, self);
   self->super.raise = do_channel_io_exception_handler;
   self->super.parent = parent;
+  self->super.context = context;
+  
   self->channel = channel;
   self->prefix = prefix;
 
