@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.77 1999/10/02 20:27:24 nisse Exp $ */
+ * $Id: client.c,v 1.79 1999/10/11 20:53:31 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -247,8 +247,7 @@ do_exit_status(struct channel_request *c,
       /* FIXME: Make this behaviour configurable, there may be some
        * child process alive that we could talk to. */
 
-      if (!(channel->flags & CHANNEL_SENT_EOF))
-	channel_eof(channel);
+      channel_eof(channel);
     }
   else
     /* Invalid request */
@@ -299,8 +298,7 @@ do_exit_signal(struct channel_request *c,
       /* FIXME: Make this behaviour configurable, there may be some
        * child process alive that we could talk to. */
 
-      if (!(channel->flags & CHANNEL_SENT_EOF))
-	channel_eof(channel);
+      channel_eof(channel);
     }
   else
     /* Invalid request */
@@ -399,12 +397,14 @@ do_client_io(struct command *s UNUSED,
   session->out->super.e
     = make_channel_io_exception_handler(channel,
 					"lsh: I/O error on stdout",
-					session->out->super.e);
+					session->out->super.e,
+					HANDLER_CONTEXT);
 
   session->err->super.e
     = make_channel_io_exception_handler(channel,
 					"lsh: I/O error on stderr",
-					session->err->super.e);
+					session->err->super.e,
+					HANDLER_CONTEXT);
 
   /* Set up the fd we read from. */
   channel->send = do_send;
