@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.33 1999/10/02 20:00:28 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.34 1999/10/24 20:06:18 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -72,7 +72,7 @@ do_handle_dh_reply(struct packet_handler *c,
   struct verifier *v;
   struct hash_instance *hash;
   
-  verbose("handle_dh_reply()\n");
+  trace("handle_dh_reply()\n");
   
   if (!dh_process_server_msg(&closure->dh, packet))
     {
