@@ -1,6 +1,6 @@
 /* digits.h
  *
- * $Id: digits.h,v 1.1 1999/06/30 20:46:35 nisse Exp $ */
+ * $Id: digits.h,v 1.2 1999/09/06 22:25:08 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -34,6 +34,7 @@ extern int hex_digits[0x100];
 #define BASE64_INVALID -1
 #define BASE64_END -2
 #define BASE64_SPACE -3
+#define BASE64_PARTIAL -4
 
 extern int base64_digits[0x100];
 
@@ -49,4 +50,10 @@ struct base64_state
 
 #define BASE64_INIT(t) {0, 0, (t)}
 
+void
+base64_init(struct base64_state *state, UINT8 terminator);
+
+int
+base64_decode(struct base64_state *state, UINT8 c);
+
 #endif /* LSH_DIGITS_H_INCLUDED */
