@@ -2,7 +2,7 @@
  *
  * user authorization database
  *
- * $Id: server_authorization.h,v 1.1 1999/09/06 22:03:50 nisse Exp $ */
+ * $Id: server_authorization.h,v 1.2 1999/10/19 17:58:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -30,7 +30,8 @@
 #include "publickey_crypto.h"
 
 struct lookup_verifier *
-make_authorization_db(struct signature_algorithm *s,
+make_authorization_db(struct lsh_string *index_name, 
+		      struct signature_algorithm *s,
 		      struct hash_algorithm *h);
 
 #endif /* LSH_SERVER_AUTHORIZATION_H_INCLUDED */
