@@ -1,6 +1,6 @@
 /* server_keyexchange.c
  *
- * $Id: server_keyexchange.c,v 1.32 1999/10/02 20:15:12 nisse Exp $ */
+ * $Id: server_keyexchange.c,v 1.33 1999/10/24 20:30:39 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -64,7 +64,7 @@ do_handle_dh_init(struct packet_handler *c,
   CAST(dh_server, closure, c);
   struct hash_instance *hash;
 
-  verbose("handle_dh_init()\n");
+  trace("handle_dh_init()\n");
   
   if (!dh_process_client_msg(&closure->dh, packet))
     {
