@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.18 1999/10/02 20:33:50 nisse Exp $
+ * $Id: server_session.c,v 1.19 1999/10/11 21:02:37 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -860,7 +860,8 @@ do_spawn_shell(struct channel_request *c,
 	  struct exception_handler *io_exception_handler
 	    = make_channel_io_exception_handler(channel,
 						"lshd: Child stdio: ",
-						&default_exception_handler);
+						&default_exception_handler,
+						HANDLER_CONTEXT);
 
 	  /* Close callback for stderr and stdout */
 	  struct close_callback *read_close_callback
