@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.2 1999/06/21 17:25:25 nisse Exp $
+ * $Id: server_session.h,v 1.3 1999/09/21 23:04:52 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -28,7 +28,7 @@
 #include "channel.h"
 #include "io.h"
 #include "reaper.h"
-#include "server_password.h"
+#include "server_userauth.h"
 
 #include <assert.h>
 #include <string.h>
