@@ -2,7 +2,7 @@
  *
  * An implementation of Ron Rivest's S-expressions, used in spki.
  *
- * $Id: sexp.h,v 1.18 1999/09/07 09:05:23 nisse Exp $ */
+ * $Id: sexp.h,v 1.20 1999/10/11 21:02:40 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -29,6 +29,7 @@
 #include "bignum.h"
 #include "command.h"
 #include "list.h"
+#include "lsh_argp.h"
 
 #include "sexp_table.h"
 
@@ -181,6 +182,10 @@ make_read_sexp(int style, int goon,
 	       struct command_continuation *c,
 	       struct exception_handler *e);
 
+extern const struct argp sexp_input_argp;
+extern const struct argp sexp_output_argp;
+#define sexp_argp_state int
+
 #endif /* LSH_SEXP_H_INCLUDED */
 
  
