@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: werror.c,v 1.37 1999/10/13 16:04:29 nisse Exp $ */
+ * $Id: werror.c,v 1.39 1999/10/25 20:11:42 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -120,8 +120,9 @@ static void write_syslog(int fd UNUSED, UINT32 length, UINT8 *data,
   syslog(LOG_NOTICE, "%s", string_buffer);
 }
 
-void set_error_syslog(void)
+void set_error_syslog(const char *id)
 {
+  openlog(id, LOG_PID | LOG_CONS, LOG_DAEMON);
   error_write = write_syslog;
   error_fd = -1;
 }
@@ -334,8 +335,19 @@ void werror_vformat(const char *f, va_list args)
 
 		while (*s)
 		  (do_paranoia ? werror_paranoia_putc : werror_putc)(*s++);
+
+		break;
+	      }
+	    case 'a':
+	      {
+		int atom = va_arg(args, int);
+		
+		assert(atom);
+
+		werror_write(get_atom_length(atom), get_atom_name(atom));
+		
+		break;
 	      }
-	      break;
 	    case 's':
 	      {
 		UINT32 length = va_arg(args, UINT32);
