@@ -2,7 +2,7 @@
  *
  * zlib compression algorithm
  * 
- * $Id: zlib.c,v 1.19 1999/08/04 06:41:29 nisse Exp $ */
+ * $Id: zlib.c,v 1.20 1999/10/02 18:31:16 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -50,7 +50,7 @@ static void do_free_zstream(z_stream *z);
        (max . UINT32)
        (rate . UINT32)
        (f pointer (function int "z_stream *" int))
-       (z special-struct z_stream
+       (z indirect-special z_stream
           #f do_free_zstream)))
 */
 
